//
//	oalserver.cc:	test OalServer class
//
#include <cassert>
#include <iostream>

#include "OalServer.h"

int main()
{
	const Odf *odf = oalServer.newOdf("../odf");

	cout<<*odf<<endl;
	assert(odf->directoryName() == "../odf");
	assert(odf->orbitNr() == 10);
	assert(odf->sets() == 84);
	assert(odf->setNames().size() == 84);
	const list<OdfSet>& ls = odf->setNames();
	for(list<OdfSet>::const_iterator k = ls.begin();
	    k != ls.end(); ++k)
	  cout << (*k) << endl;

	oalServer<<"../odf/0010_PPPPPPOOLL_PNS00104IME.FIT";
	cout<<"Oal state set to:\n"
		<<"   instrument             : "<<oalServer.instrument()<<endl
		<<"   data mode              : "<<oalServer.dataMode()<<endl
		<<"   exposure index         : "<<oalServer.exposureIndex()<<endl
		<<"   exposure number        : "<<oalServer.exposureNumber()<<endl
		<<"   ccd number             : "<<oalServer.ccd()<<endl
		<<"   node number            : "<<oalServer.node()<<endl;
	assert(oalServer.instrument() == Xmm::EPN);
	assert(oalServer.dataMode() == Xmm::IMAGING);
	assert(oalServer.exposureIndex() == 1);
	assert(oalServer.ccd() == 4);
	assert(oalServer.node() == 0);

	const Exposure* ex = odf->currentExposure();
	const IPPVCollection &iv = ex->ippvCollection();
	for(IPPVCollection::const_iterator i = iv.begin();
		i != iv.end(); ++i)
		cout<<i->first<< " -> "<<i->second<<endl;
	assert(iv.size() == 2);
	assert(ex->ippvCollection().hasIPPV("MODE"));
	assert(ex->ippvCollection().hasIPPV("FILTER"));
	// The summary file contains MODE = FULL FRAME
	//	assert(ex->getIPPV("MODE").value() == "FULL FRAME");
	assert(ex->ippvCollection().getIPPV("MODE").value() == "FULL");
	assert(ex->ippvCollection().getIPPV("FILTER").value() == "OPEN");

	return 0;
}

