//
//	timecorrfit: simple program to evaluate quality of OBT-MET fit
//
//	For each data point (OBT/MET) in the TCS we print:
//		* the difference between the MET and the fit polynomial evaluated at
//		  OBT
//		* the evelaction of the satellite over all the groundstations
//

#include <iostream>
#include <cmath>
#include <vector>
#include "TimeCorrelator.h"
#include "PositionLocator.h"
#include "OalServer.h"
#include "Xmm.h"
#include "Celestial.h"

int main()
{
	const TimeCorrelator *tcorr = oalServer.timeCorrelator();
	const XmmPositionLocator *posloc = oalServer.positionLocator();

	const FitPolynomial *poly = tcorr->obt2MetPoly();
	const vector<double> &OBT = poly->xPoints(),
						 &MET = poly->yPoints();

	cout.precision(12);
	unsigned N = OBT.size();

	//	print OBT-MET pairs
	for (unsigned n=0; n<N; ++n)
		cout<<OBT[n]<<' '<<MET[n]-poly->evaluate(OBT[n])<<endl;
//		cout<<OBT[n]<<' '<<MET[n]-(OBT[n]+(MET[0]-OBT[0]))<<endl;
//		cout<<OBT[n]<<' '<<poly->firstDerivative(OBT[n])<<endl;

	//	for all ground stations - compute elevation of satellite vs time
	cout<<endl;
	for (unsigned p=0; p <= XmmMission::Kourou; ++p) {
		const XmmGroundStation &gs = XmmMission::groundStation(
			XmmMission::GroundStationPlaces(p));
		cout<<gs.earthObject().name()<<':'<<endl;
		tcorr->resetObtMetRelation();
		for (unsigned n=0; n<N; ++n) {
			double met = tcorr->toMissionEllapsedTime(OBT[n]);
			try {
				CartesianVector xmmPos = posloc->stateVector(met).position,
								gsPos = gs.stateVector(met).position,
								delta = xmmPos - gsPos;	//	GS -> SC
				delta.normalize();	//	unit vector
				gsPos.normalize();	//	unit vector
				cout<<OBT[n]<<' '<<90.-RAD_TO_DEG*acos(delta * gsPos)<<endl;
			} catch(XmmPositionLocator::NoPositionDataAvailable) {
				cout<<OBT[n]<<" 0.\n";
			}
		}
	}
	cout<<endl;

	return 0;
}

