//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	TimeTags.h	--- classes for CCD frame counters and time tags
//
//	DESCRIPTION:
//		Classes representing CCF frame counters and time tags are defined.
//
//	CHANGE HISTORY:
//		UL, 1998-08-03: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Mon Aug  3 14:31:48 MET DST 1998
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/oal/src/TimeTags.h,v 1.4 2002/01/21 16:11:34 ulammers Exp $
//

#ifndef TIME_TAGS_H
#define TIME_TAGS_H

#include <vector>

#include "Dal.h"
#include "Tracer.h"
#include "OalChild.h"
#include "Xmm.h"

class OalServer;
class ObtVector;

class FrameCounterVector : public virtual OalChild<OalServer> {
public:
	FrameCounterVector(const OalServer *oalEngine = 0)
		: OalChild<OalServer>(oalEngine) {}

	virtual ObtVector toObt() const = 0;

	virtual ~FrameCounterVector() {}
};


template<class Instr>
class InstrFrameCounterVector : public FrameCounterVector,
	public vector<int32> {
public:
	InstrFrameCounterVector(const int32 counterVec[], unsigned nCounter,
							const OalServer *oalEngine = 0)
		: OalChild<OalServer>(oalEngine), FrameCounterVector(oalEngine)
	{
		this->reserve(nCounter);
		for (unsigned n=0; n<nCounter; this->push_back(counterVec[n++]));
	}

	static void resetWraps() { _nWraps = 0; _lastWrapAt = 0; }
	static void resetResetTimes() { _resetTimes.clear(); }
	static void reset() { resetWraps(); resetResetTimes(); }

	ObtVector toObt() const;

	~InstrFrameCounterVector() {}

private:
	//
	//	data
	//
	static unsigned _nWraps,		//	number of instrument clock
					_lastWrapAt;	//	wrap-arounds and index of last
									//	wrap around
	static std::vector<double> _resetTimes;
									//	time of instrument clock reset
									//	values at which coarse time wraps
									//	around
	static unsigned _wrapValue;		//	FTCOARSE wraps around at this value
	static unsigned short _chopperValue;
									//	only every 1+_chopperValue frame
									//	is registered in AUX data

	//
	//	methods
	//
	void setStatics() const;		//	check if reset time/warp value
									//	are defined
};



class ObtVector : public virtual OalChild<FrameCounterVector>,
	public std::vector<double> {
public:
	ObtVector(const FrameCounterVector *frameCounterVector = 0)
		: OalChild<FrameCounterVector>(frameCounterVector) {}

	std::vector<double> toMET() const;

	~ObtVector() {}
};

#endif
